#!/bin/sh

# Copyright (C) 2004,2005 Nokia

# This is a wrapper for make_rootimage.sh
# It scans configuration directories and builds required 
# fs and rootstrap images

# Parameters: 
# $1 - configuration directory (optional)

die(){ echo "$1"; exit 1;}

usage()
{
	echo "Usage:"
	echo "`basename $0` [configuration]"
	echo " configuration (optional) - configuration directory to be processed"
	exit 0
}

[ '-h' = "$1" -o '-H' = "$1" -o '--help' = "$1" -o '/?' = "$1" ] && usage

if [ -f /targets/links/scratchbox.config ] ; then
        . /targets/links/scratchbox.config
else
        die "ERROR: this script should be run inside scratchbox"
fi

HOST_ARCH=`arch`
echo $HOST_ARCH | grep -q 'i[3-6]86' && HOST_ARCH=i386

UPDIR='.'
[ -n "$1" ] && UPDIR=`dirname $1`

[ -d "$UPDIR" ] || die "ERROR: "$UPDIR" is not a directory"

if [ -n "$1" ] ; then
	pfiles="`dirname $1`/`basename $1`/config"
	[ -r "$pfiles" ] || die "Cannot read $pfiles file"
else
	pfiles=`find $UPDIR -mindepth 1 -maxdepth 2 -name config`
	[ -z "$pfiles" ] && die "ERROR: There are no config files in the subdirectories of $path"
fi

for f in $pfiles ; do
	
	EXTRAOPTS=''
	
	. $f
	
	DIR=`dirname $f`
	CONF=`basename $(dirname $f)`

	if [ "$HOST_ARCH" != `echo $CONF |cut -f1 -d-` ] ; then
		echo "Skipping $CONF: wrong arch (should be $HOST_ARCH)"
		continue
	fi	

	echo "Creating $CONF image"

        [ -r $DIR/packages ] || die "Cannot read $DIR/packages file"

	CMDLINE="--packages=$DIR/packages"
	if [ -r $UPDIR/version ] ; then
		CMDLINE="$CMDLINE --version $DIR/version"
	fi
	[ -r ${DIR}/removes ] && CMDLINE="$CMDLINE --removes=$DIR/removes"
	
	if [ -r ${DIR}/sources.list ] ; then
		CMDLINE="$CMDLINE --srclist=${DIR}/sources.list"
	elif [ -r ${UPDIR}/sources.list ] ; then
		CMDLINE="$CMDLINE --srclist=${UPDIR}/sources.list"
	else
		die "ERROR: There are no ${DIR}/sources.list nor ${UPDIR}/sources.list"
	fi
	
	CMDLINE="$CMDLINE --outprefix=$DIR/$CONF $EXTRAOPTS"

	echo "make_rootimage.sh $CMDLINE"
	make_rootimage.sh $CMDLINE 2>&1 | tee $DIR/$CONF.log || die "Error building $CONF image" && echo "Done $CONF"
done

