#!/bin/sh
# vim: sw=4 ts=4 expandtab ai

if [ -r /targets/links/scratchbox.config ]
then
    . /targets/links/scratchbox.config
else
    echo "this script needs to be run inside scratchbox"
fi

if [ $UID != 0 ]
then
    echo "this script needs to be run under fakeroot!"
    exit 1
fi

usage()
{
	echo "`basename $0` v.0.0.1"
	echo "Usage:"
	echo "fakeroot `basename $0` [options] -o <image_name>"
	echo "Converts your current scratchbox target to a rootimage"
	echo "Available options:"
	echo " -o, --out       - output file prefix (mandatory)"
	echo " -u, --udev      - re-install udev" 
	echo " -r, --rootfs    - make rootfs tarball"
	echo " -s, --rootstrap - make rootstrap tarball"
	echo " -j, --jffs2     - make jffs2 image"
	exit 1
}

TEMP=`getopt -o o:ursj --long out:,udev,rootfs,rootstrap,jffs2 -- "$@"`

if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi

# Note the quotes around `$TEMP': they are essential!
eval set -- "$TEMP"

while true ; do
	case "$1" in
		-o|--out)
			IMAGE=$2
			shift 2
			;;
		-u|--udev)
			UDEV="Y"
			shift
			;;
		-r|--rootfs)
			ROOTFS="Y"
			shift
			;;
		-s|--rootstrap)
			ROOTSTRAP="Y"
			shift
			;;
		-j|--jffs2)
			JFFS="Y"
			shift
			;;
		--) 
			shift
			break 
			;;
		*)
			echo "Internal getopt error"
			exit 1
			;;
	esac
done

if [ x$IMAGE = x ] ; then
    echo -e "\nERROR: -o,--out is a mandatory option\n"
    usage;
fi

if [ x$ROOTFS = x -a x$ROOTSTRAP = x -a x$JFFS = x ] ; then
    echo "Nothing to do. At least one of --rootstrap, --rootfs or --jffs2 options must be set"
    exit 1;
fi

mkdir -p /targets/$SBOX_TARGET_NAME/dev

if [ x$UDEV = xy ] ; then
    apt-get install -y --reinstall makedev udev
else
    apt-get install -y --reinstall makedev
fi

if [ $? != 0 ] ; then
    echo "apt-get failed!!"
    exit 1
fi

apt-get clean

if [ x$ROOTFS = xy -o x$ROOTFS = xY ] ; then
    echo "Making rootfs TAR"
    mv /usr/sbin/invoke-rc.d_ORIG /usr/sbin/invoke-rc.d || true
    tar cfz $IMAGE-rootfs.tgz -C /targets/$SBOX_TARGET_NAME .
fi

if [ x$JFFS = xy -o x$JFFS = xY ] ; then
    echo -n "Making JFFS2 images: "
    if which mkfs.jffs2 > /dev/null; then
        echo -n "$IMAGE-128.jffs2 "
        export SBOX_ENV_LD_PRELOAD=/scratchbox/tools/lib/libfakeroot-tcp.so.0
        export LD_LIBRARY_PATH=/scratchbox/host_shared/lib
        /scratchbox/host_shared/lib/ld-linux.so.2 /scratchbox/tools/bin/mkfs.jffs2 \
            -d /targets/$SBOX_TARGET_NAME -e 128 -l -n -o $IMAGE-128.jffs2.tmp
        /scratchbox/host_shared/lib/ld-linux.so.2 /scratchbox/tools/bin/sumtool -i $IMAGE-128.jffs2.tmp \
            -o $IMAGE-128.jffs2 -e 128KiB -l -n
        rm -f $IMAGE-128.jffs2.tmp
        echo "done."
    else
        echo "mkfs.jffs2 NOT INSTALLED"
        exit 1
    fi
fi

if [ x$ROOTSTRAP = xy -o x$ROOTSTRAP = xY ] ; then
    echo "cleaning /dev"
    rm -rf /targets/$SBOX_TARGET_NAME/dev/*
    rm -rf /targets/$SBOX_TARGET_NAME/.dev/*
    #remove possibly sensitive info
    echo "Making rootstrap TAR"
    tar cf $IMAGE.tar -C /targets/$SBOX_TARGET_NAME .
    tar --delete -f $IMAGE.tar ./etc/hosts ./etc/resolv.conf
    gzip $IMAGE.tar
    mv $IMAGE.tar.gz $IMAGE-rootstrap.tgz
fi


