#!/bin/sh

if [ -n "$1" ]
then
	. $1
fi

basedir=`dirname $0`

# Get the environment variable we set in the previous script

#. .makeroot_env

# rm -f .makeroot_env

. /targets/links/scratchbox.config


DEBIAN_FRONTEND=noninteractive
export DEBIAN_FRONTEND

export TMPDIR=/var/tmp
export TMP=$TMPDIR

#
# suppress xargs bug
#
unset PACKAGES

if [ x$SBOX_CPUTRANSPARENCY_METHOD != xfalse ]; then
	# Try to configure all packages on the host system (fast)
	echo "Configuring all packages on host"
	chmod 666 /etc/hosts
	dpkg --configure -a

	# Try to configure the rest of the packages on the target system (slow)
	echo "Configuring all packages on target"
	# chmod 666 /etc/hosts
	export SBOX_REDIRECT_IGNORE=/usr/bin/dpkg
	/usr/bin/dpkg --configure -a
else
	echo "x86"
	unset SBOX_REDIRECT_IGNORE
	fakeroot dpkg --configure -a
	dpkg --configure -a	
fi
	
set -e
echo "Configuring all packages on target second time"

/usr/bin/dpkg --configure -a

#clean traces of users away
if [ -r "$SRCLIST" ] ; then
	apt=`tempfile`
	grep -v workaround /etc/apt/sources.list > $apt
	mv $apt /etc/apt/sources.list
else
	echo > /etc/apt/sources.list
fi

rm -rf $builddir
mkdir -p $builddir 

cp /var/lib/dpkg/info/*list $builddir

if [ -n "$REMOVES" ]
then
	echo "Packages '$REMOVES' will be removed."
	set +e
	for package in $REMOVES
	do
		echo -n "Removing $package.. : "
		dpkg --purge --force-all $package  2>&1 | grep -q 'ignoring request to remove'
		if [ $? -eq 0 ]
		then
			echo "was not installed, so wasn't removed"
		else
			echo "done."
		fi
	done
	set -e
fi

if [ -n "$STRIP" ] ; then
	
	for i in `dpkg --get-selections|sed -e 's/\t.*//'`
	do
		if [ -x $basedir/strip.d/$i ]	
		then
			echo Stripping $i with $basedir/strip.d/$i
			$basedir/strip.d/$i
		fi
	done
fi 

if [ -n "$PRELINK" -a -x /usr/sbin/prelink ] ; then
	echo "prelinking: "
	/usr/sbin/prelink -a -v
	echo " done"
	dpkg --purge --force-all prelink libelfg0
	echo "removed prelink"
fi

#env

if [ x$SBOX_CPUTRANSPARENCY_METHOD = xfalse ]; then
    echo "fakeroot hack No 2 for x86 target (start)"
    LF=$LD_PRELOAD
    cp $LF $LF.bak 
fi

set +e
echo "Trying to remove fakeroot: "
if dpkg -r fakeroot 
then
	echo "fakeroot removed"
else
	echo "fakeroot not removed"
fi

echo "Trying to remove fakeroot: "
if dpkg -r fakeroot-net 
then
	echo "fakeroot-net removed"
else
	echo "fakeroot-net not removed"
fi
set -e

if [ x$SBOX_CPUTRANSPARENCY_METHOD = xfalse ]; then
    echo "fakeroot hack No 2 for x86 target (end)"
    LD_PRELOAD= mv $LF.bak $LF
fi


#if [ x$NODEV = xy -o x$NODEV = xY ] ; then	
#	echo "Not removing fakeroot-net, needed on rootstraps"
#else
#	echo "Removing fakeroot"
#	dpkg --purge --force-all fakeroot fakeroot-net
#fi

if [ -n "$STRIP" ] ; then
	export STRIP
	$basedir/strip.sh #general strip
fi

#
# removed due to the strange problems with fakeroot
#
#if [ -x $basedir/dlocate.sh ] ; then
#	echo -n "Generating dlocate db: "
#	$basedir/dlocate.sh $builddir > $STAMP.csv
#	echo "done"
#fi
#rm -rf $builddir

rm -f /root/.sbrsh

if getent passwd messagebus > /dev/null 2>&1 ; then 
	mkdir -p /var/lib/gconf/
	chown messagebus:messagebus -R /var/lib/gconf/
fi

if [ x$REMOVEUSER != x ] ; then
	echo "cleaning user $REMOVEUSER from passwd and group"
	mv /targets/$SBOX_TARGET_NAME/etc/passwd passwd.1
	mv /targets/$SBOX_TARGET_NAME/etc/group group.1
	grep -v $REMOVEUSER passwd.1 > /targets/$SBOX_TARGET_NAME/etc/passwd
	grep -v $REMOVEUSER group.1 > /targets/$SBOX_TARGET_NAME/etc/group
fi

if [ x$ROOTFS = xy -o x$ROOTFS = xY ] ; then
	echo "Making rootfs TAR"
        mv /usr/sbin/invoke-rc.d_ORIG /usr/sbin/invoke-rc.d || true
        tar cfz $STAMP-rootfs.tgz -C /targets/$SBOX_TARGET_NAME .
fi

if [ x$ROOTSTRAP = xy -o x$ROOTSTRAP = xY ] ; then
	echo "cleaning /dev"
	rm -rf /targets/$SBOX_TARGET_NAME/dev/*
	rm -rf /targets/$SBOX_TARGET_NAME/.dev/*
	#remove possibly sensitive info
	echo "Making rootstrap TAR"
	tar cf $STAMP.tar -C /targets/$SBOX_TARGET_NAME .
	tar --delete -f $STAMP.tar ./etc/hosts ./etc/resolv.conf
	gzip $STAMP.tar
	mv $STAMP.tar.gz $STAMP-rootstrap.tgz
fi

# Create tar.gz rootimage

if [ x$JFFS = xy -o x$JFFS = xY ] ; then
	echo -n "Making JFFS2 images: "
	if which mkfs.jffs2 > /dev/null; then
		echo -n "$STAMP-128.jffs2 "
		export SBOX_ENV_LD_PRELOAD=/scratchbox/tools/lib/libfakeroot-tcp.so.0 
		export LD_LIBRARY_PATH=/scratchbox/host_shared/lib 
		/scratchbox/host_shared/lib/ld-linux.so.2 /scratchbox/tools/bin/mkfs.jffs2 -d /targets/$SBOX_TARGET_NAME -e 128 -l -n -o $STAMP-128.jffs2.tmp
		/scratchbox/host_shared/lib/ld-linux.so.2 /scratchbox/tools/bin/sumtool -i $STAMP-128.jffs2.tmp -o $STAMP-128.jffs2 -e 128KiB -l -n
		rm -f $STAMP-128.jffs2.tmp
		echo "done."
	else
		echo "mkfs.jffs2 NOT INSTALLED"
		exit 1
	fi
fi

#recovering old passwd and group for convinience

mv -f passwd.1 /etc/passwd
mv -f group.1 /etc/group

