#!/bin/sh

function usage()
{
	cat <<EOF
Usage:
$0 <temp_dir> <deb_dir>

$0 will extract all deb packages found in <deb_dir> to the
<temp_dir>

EOF
exit 1;
}


function error()
{
	echo "ERROR: "$@;
	exit 1;
}

function status()
{
	echo -ne "STATUS: "$@
}

if [ $# != 2 ] ; then 
	usage;
fi

debs_found=false;

for i in $2/*.deb ; do
	if [ -f $i ] ; then 
		debs_found=true;
		break;
	fi
done

if ! $debs_found ; then 
	error "no deb's found in \"$2\"";
fi 

if ! which dpkg-deb>/dev/null 2>&1 ; then
	error "you must install \"debian\" devkit. see sbox-config";
fi

status "making some essential directories...\n";

mkdir -p $1/var/lib/dpkg/info 2>/dev/null
mkdir $1/tmp 2>/dev/null
mkdir $1/var/lib/dpkg/alternatives 2>/dev/null
mkdir $1/var/lib/dpkg/methods 2>/dev/null
mkdir $1/var/lib/dpkg/parts 2>/dev/null
mkdir $1/var/lib/dpkg/updates 2>/dev/null

touch $1/var/lib/dpkg/status
touch $1/var/lib/dpkg/available

mkdir -p $1/var/backups 2>/dev/null
echo "# this file cannot be empty (dpkg bug)" > $1/var/backups/infodir.bak

mkdir -p $1/etc 2>/dev/null
echo -n "Europe/Helsinki" > $1/etc/timezone

mkdir -p $1/var/cache/debconf 2>/dev/null 
cat <<EOF > $1/var/cache/debconf/config.dat
Name: debconf/frontend
Template: debconf/frontend
Value: Noninteractive
Owners: debconf
Flags: seen

Name: debconf/priority
Template: debconf/priority
Value: critical
Owners: debconf
Flags: seen

Name: dash/sh
Template: dash/sh
Owners: dash
Value: true
Flags: seen

EOF

status "extracting debs";

for i in $2/*.deb ; do 
	name=`dpkg-deb -f $i Package`
	echo -n "."
	dpkg-deb -x $i $1
	dpkg-deb -e $i $1/tmp
	dpkg-deb -c $i | 
	awk '{print $6}' | sed 's/^\.//;s/^\/$/\/./;s/\/$//' > $1/var/lib/dpkg/info/$name.list
	for j in $1/tmp/* ; do
		mv $j $1/var/lib/dpkg/info/$name.${j/*\/}
	done	
done
echo

status "modifying status fields...\n"

for i in $1/var/lib/dpkg/info/*.control ; do
	awk -v dir=$1 'function print_line(line) {
if(lines[line])
print lines[line];
}
/^Package:/ { name=$2 }
/^[A-Z]/ { lines[$1]=$0 }
/^ /     { lines["Description:"]=(lines["Description:"] "\n" $0) }
END      {
print_line("Package:");
print_line("Essential:");
print "Status: install ok unpacked"
print_line("Priority:");
print_line("Section:");
print_line("Installed-Size:");
print_line("Maintainer:");
print_line("Architecture:");
print_line("Source:");
print_line("Version:");
print_line("Replaces:");
print_line("Provides:");
print_line("Depends:");
print_line("Pre-Depends:");
print_line("Recommends:");
print_line("Suggests:");
print_line("Conflicts:");
print_line("Description:");
print "";
}' $i >> $1/var/lib/dpkg/status;
	rm $i;
done

status "ready...\n"


