#!/bin/sh
# vim: sw=4 ts=4 expandtab ai

basedir=`dirname $0`

if [ -d $basedir/../share/rootimagescripts/ ]
then
	tooldir=$basedir/../share/rootimagescripts/
else
	tooldir=$basedir
fi

possible_strip_options="all apt debian docs locales zoneinfo x11"
strip_options=""

usage_message="Copyright (C) 2004 Nokia
Licensed under GNU General Public License (GPL) version 2
Usage: $0 [--strip=] [COMMAND]

  -s OPTION --strip=OPTION         strip the OPTION parts of packages. 
                                   supported strip options 
                                   are: $possible_strip_options
  -r REMOVE --removes=REMOVE        REMOVE is a file containing
                                   a list of packages that will be
				   forcefully removed
  -p PACKAGES --packages=PACKAGES  PACKAGES is a file conainen a list
  				   packages that will be included in
				   the rootstrap/rootimage
  -o PREFIX --outprefix=PREFIX     Put result files into 
                                   PREFIX.[tgz|jffs2|csv|txt] 
  -j --jffs2			   create a jffs2 image also
  -a --save-archive		   save dowloaded debs
  -l --genlists                    generate list of binary and source packages
  -f --footstat			   generate footprint statistics (.csv)
  --rootstrap		   	   Create rootstrap
  --rootfs			   Create rootfs
  --srclist=SRCLIST		   source.list location
  -v --version			   Add a version string or a file
 
Examples:
  $0 --strip=all --removes=./removelist --packages=./install-list
  $0 --p=./development-package-list -s \"docs locales\"
"

export STAMP=`date +%Y%m%d-%H%M%S`

TEMP=`getopt -o p:s:ajr:o:dgfhxl:v: \
 --long packages:,strip:,save-archive,jffs2,removes:,outprefix:,rootstrap,rootfs,srclist:,genlists,footstat,help,pref:,prelink,version: \
 -- "$@"`

if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi

# Note the quotes around `$TEMP': they are essential!
eval set -- "$TEMP"

while true ; do
    case "$1" in
        -p|--packages)      
            echo "Packages file: $2 " 
            export PACKAGES="fakeroot-net `cat $2 | tr '\n' ' '`"
            shift 2 
            ;;
        -s|--strip)         
            strip_options="$strip_options $2"
            export STRIP="$strip_options"
            echo "Strip options: $STRIP" 
            shift 2
            ;;
        -a|--save-archive)  
            echo "Save archives: yes" 
            save_archive="Y"
            shift  
            ;;
        -j|--jffs2)         
            echo "Create jffs2 : yes" 
            export JFFS="Y"
            shift 
            ;;
        -r|--removes)       
            echo "Removes file : $2 " 
            export REMOVES=`cat $2 | tr '\n' ' '`
            shift 2
            ;;
        -o|--outprefix)
            echo "Output prefix: $2 " 
            export STAMP=$2
            shift 2
            ;;
        -g|--genlists)         
            echo "Package lists : yes" 
            export PKGLISTS="Y"
            shift
            ;;
        -f|--footstat)
            echo "Footprint statistics : yes"
            export FOOTSTAT="Y"
            shift
            ;;
        --rootstrap)         
            echo "rootstrap    : yes"
            export ROOTSTRAP="Y"
            shift
            ;;
        --rootfs)
            echo "rootfs    : yes"
            export ROOTFS="Y"
            shift
            ;;
        --srclist)	
            echo "srclist	: $2"
            export SRCLIST=$2
            shift 2
            ;;
        -h|--help)          
            echo "$usage_message"
            exit
            ;;
        -v|--version)
            if [ -r "$2" ] 
            then 
                echo "version file  : $2"
                export VERSION=`cat $2`
            else 
                echo "version       : $2"
                export VERSION=$2
            fi
            shift 2
            ;;
        -l|--prelink)
            echo "Prelink: yes"
            export PACKAGES="$PACKAGES prelink"
            export PRELINK="Y"
            shift 
            ;;
        --) shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done

if [ -z "$PACKAGES" ]
then
	echo "mandatory option --packages missing"
	echo "$usage_message"
	exit 1
fi

if [ ! -r /targets/links/scratchbox.config ] ; then
	echo "This script needs to be run from inside scratchbox"
	exit -1
else
	. /targets/links/scratchbox.config 
fi

if [ "$SBOX_CPUTRANSPARENCY_METHOD" = "sbrsh" ] ; then
	echo "Unmounting mounts possible mounts"
	sbrsh --umount-all $SBOX_TARGET_NAME
fi

set -e

echo "Resetting target $SBOX_TARGET_NAME"
sbox-config -rt $SBOX_TARGET_NAME

echo "Copying fakeroot"
sbox-config -cf > /dev/null 2>&1 || true

mkdir -p /targets/$SBOX_TARGET_NAME/dev/pts

if [ ! -e /targets/links/devkit_bin ] ; then
	# select debian devkit
	sbox-config -sd debian > /dev/null || true
fi
		
echo "Creating directories for apt..."

mkdir -p /etc/apt /var/lib/apt/lists/partial/ /var/cache/apt/archives/partial/ /var/lib/dpkg/ /var/lib/xml-core /etc/xml && touch /etc/apt/sources.list /var/lib/apt/lists/lock /var/lib/dpkg/lock /var/lib/dpkg/status /var/lib/dpkg/diversions 

if [ -r apt.conf ]
then
	cp apt.conf /etc/apt/
fi

[ -z "$SRCLIST" ] && export SRCLIST=sources.list

if [ -r "$SRCLIST" ]
then
	echo "Using provided sources.list"
    	cp $SRCLIST /etc/apt/sources.list
else
	echo > /etc/apt/sources.list
fi

if [ -r preferences ]
then
        echo "Copying preferences to /etc/apt"
	cp preferences /etc/apt/
fi

if [ -d workarounds ]
then
	echo "adding workaround dir"
	echo "deb file:$PWD/workarounds ./" >> /etc/apt/sources.list
	cp workarounds/*deb /var/cache/apt/archives/ || true 
	(
		cd workarounds
		dpkg-scanpackages . /dev/null |gzip > Packages.gz
	)
fi

echo "Updating apt package cache..."
apt-get update

echo "Creating image from: $PACKAGES"

apt-get -q -d -y install $PACKAGES

echo "... done downloading."

if [ x$SAVE_ARCHIVE = xy -o  x$SAVE_ARCHIVE = xY ]
then
	echo "Storing used debs into $STAMP-repository.tgz"
	tar czf "$STAMP-repository.tgz" -C /var/cache/apt/archives .
fi

if [ x$PKGLISTS = xy -o x$PKGLISTS = xY ]
then
	# Generate package lists
	echo "Creating lists of binary/source packages: "
	echo -n "$STAMP-pkglist.txt ... "
	ls /var/cache/apt/archives/*deb | sed s,/var/cache/apt/archives/,, |cut -d_ -f1,2|sed -e 's/_/ /' -e 's/%3a/:/' > $STAMP-pkglist.txt && echo "Done" || echo "Failed"
	echo -n "$STAMP-srclist.txt ... "
	$tooldir/gensrclist.sh $STAMP-pkglist.txt|sort|uniq > $STAMP-srclist.txt && echo "Done" || echo "Failed"

	echo 
fi

echo "Extracting packages to /targets/$SBOX_TARGET_NAME"
$tooldir/droot.sh /targets/$SBOX_TARGET_NAME /var/cache/apt/archives

# Dont start stuff
if [ -f /usr/sbin/invoke-rc.d ] ; then
	cp /usr/sbin/invoke-rc.d /usr/sbin/invoke-rc.d_ORIG
	echo "#!/bin/sh" >  /usr/sbin/invoke-rc.d
	chmod a+x /usr/sbin/invoke-rc.d
fi

cp $tooldir/gconftool-wrapper /host_usr/bin/gconftool-2

export SBOX_REDIRECT_BINARIES=$SBOX_REDIRECT_BINARIES,/usr/bin/gconftool-2:/host_usr/bin/gconftool-wrapper
export PATH=/host_usr/bin/:$PATH

if [ -x /usr/sbin/update-passwd ]
then
	echo "Setting up passwd & groups"
	update-passwd
else
	echo "Something wrong with your packages list!!!"
	exit 1
fi

export REMOVEUSER=`whoami`

echo "Clearing package cache"
apt-get clean

if [ ! -n "$VERSION" ]
then
	VERSION="custom_"`date +%V`
fi

cat > /etc/software_version <<EOF
$VERSION
EOF

if [ "$SBOX_CPUTRANSPARENCY_METHOD" = "sbrsh" ] ;then
	cp -f ~/.sbrsh /root/.sbrsh
fi

builddir=/tmp/$SBOX_TARGET_NAME/$STAMP
export builddir

# workaroung for xargs bug
unset PACKAGES

if [ x$SBOX_CPUTRANSPARENCY_METHOD = xfalse ]; then
	echo "fakeroot hack for x86 target"
	export SBOX_REDIRECT_IGNORE=/usr/bin/faked:/usr/bin/fakeroot
	/usr/bin/fakeroot $tooldir/conf.sh #$PREFERENCES
else
	fakeroot $tooldir/conf.sh #$PREFERENCES
fi

if [ x$FOOTSTAT = xy -o x$FOOTSTAT = xY ]
then
	echo "Generating footprint statistics .csv: "
	# generate dlocate db
	if [ -x $tooldir/dlocate.sh ] ; then
		echo -n "Generating dlocate db: "
		$tooldir/dlocate.sh $builddir > $STAMP.csv
		echo "done"
	else
		echo "There is no $tooldir/dlocate.sh"
		exit 1
	fi
fi
rm -rf $builddir

# ... But let stuff get started _afterwards_
if [ -f /usr/sbin/invoke-rc.d_ORIG ] ; then
	mv /usr/sbin/invoke-rc.d_ORIG /usr/sbin/invoke-rc.d
fi
rm -f /host_usr/bin/gconftool-2


echo "DONE!!"
